import json

from django.conf import settings
from sorl.thumbnail import get_thumbnail


class Image(dict):
    def __init__(self, item):

        try:
            preview_url = get_thumbnail(item["file"], "100x100").url
        except Exception:
            preview_url = None

        super().__init__({
            **item,
            "url": settings.MEDIA_URL + item["file"],
            "preview_url": preview_url,
        })

    def __hash__(self):
        return hash(self["uuid"])


class ImageListObject:
    def __init__(self, items):
        self.items = [Image(item) for item in items]

    def __repr__(self):
        return json.dumps(self.items)

    def __iter__(self):
        return iter(self.sorted_items)

    def __len__(self):
        return len(self.items)

    @property
    def sorted_items(self):
        return list(sorted(self.items, key=lambda x: x.get("order", 0)))

    @property
    def logo(self):
        items = self.sorted_items
        if items:
            return items[0]["file"]
        return None



import json
import uuid
from datetime import datetime

from django import forms
from django.core.files.storage import default_storage
from django.template.loader import render_to_string

from attachments.utils import ImageListObject


class ImageListWidget(forms.HiddenInput):

    def render(self, name, value, attrs=None, renderer=None):
        html = super().render(name, value, attrs, renderer)
        html += render_to_string(
            "attachments/field.html",
            {
                "field_id": attrs["id"],
                "value": value
            },
        )
        return html

    class Media:
        css = {
            'all': [
                'attachments/file-manager.css',
                'dropzone/dropzone.min.css'
            ]
        }
        js = [
            'dropzone/dropzone.min.js',
            'attachments/file-manager.js',
        ]


class ImageListFormField(forms.CharField):

    def __init__(self, *args, **kwargs):
        super().__init__(
            widget=ImageListWidget,
            required=False,
            *args,
            **kwargs
        )

    def clean(self, value):
        if not value:
            return []
        try:
            items = json.loads(value)
        except json.JSONDecodeError:
            raise forms.ValidationError("Invalid JSON format")

        return ImageListObject([self._clean_item(item) for item in items])

    def _clean_item(self, item):

        if not item.get("uuid"):
            raise forms.ValidationError("No UUID in item")

        try:
            uuid.UUID(item["uuid"])
        except ValueError:
            raise forms.ValidationError("Invalid UUID value")

        if not item.get("file"):
            raise forms.ValidationError("No file in item")

        if not default_storage.exists(item["file"]):
            raise forms.ValidationError("File not found: " + item["file"])

        try:
            int(item["order"])
        except ValueError:
            raise forms.ValidationError("Invalid order value")

        if not item.get("created"):
            raise forms.ValidationError("No creation timestamp in item")

        try:
            datetime.fromisoformat(item["created"])
        except ValueError:
            raise forms.ValidationError("Invalid timestamp value")

        return {
            "file": item["file"],
            "order": item["order"],
            "created": item["created"],
            "uuid": item["uuid"],
        }


class UploadImageForm(forms.Form):

    url = forms.URLField(required=False)
    file = forms.ImageField(required=False)



import uuid
from datetime import datetime
from urllib.error import HTTPError
from urllib.request import urlretrieve

from django.core.files import File
from django.core.files.storage import default_storage
from django.http.response import JsonResponse, HttpResponseBadRequest
from django.contrib.admin.views.decorators import staff_member_required
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from django.utils.translation import gettext_lazy as _

from attachments.utils import Image
from attachments.forms import UploadImageForm


@csrf_exempt
@require_POST
@staff_member_required
def upload_image(request):

    form = UploadImageForm(data=request.POST, files=request.FILES)

    if form.is_valid():

        data = form.cleaned_data

        if data.get('url'):
            try:
                result = urlretrieve(data['url'])
                file = File(open(result[0], 'rb'))
            except HTTPError as e:
                error = _('Can not upload {}: {}').format(data['url'], str(e))
                return HttpResponseBadRequest(error)

        elif data.get('file'):
            file = data['file']

        else:
            return HttpResponseBadRequest('No data')

        file_uuid = str(uuid.uuid4())
        ext = file.name.split('.')[-1]
        file_name = f'tmp/{file_uuid}.{ext}'
        default_storage.save(file_name, file)

        return JsonResponse(
            Image({
                'uuid': file_uuid,
                'file': file_name,
                'created': datetime.now().isoformat(),
            })
        )

    return HttpResponseBadRequest('Data not valid')



import shutil

from django.contrib.postgres.fields import ArrayField
from django.core.files.storage import default_storage
from django.db.models import JSONField

from attachments.forms import ImageListFormField
from attachments.utils import ImageListObject


class ImageListField(ArrayField):
    def __init__(self, verbose_name=None, upload_to=None, **kwargs):
        self.upload_to = upload_to
        super().__init__(
            JSONField(
                blank=True,
                null=True,
                default=dict,
            ),
            verbose_name=verbose_name,
            blank=True,
            null=True,
        )

    def formfield(self, **kwargs):
        return ImageListFormField(label=self.verbose_name, **kwargs)

    def _from_db_value(self, value, expression, connection):
        if value is None:
            return ImageListObject([])
        items = [
            self.base_field.from_db_value(item, expression, connection)
            for item in value
        ]
        return ImageListObject(items)

    def get_db_prep_value(self, value, connection, prepared=False):
        if value is None:
            return value
        return super().get_db_prep_value(value.sorted_items, connection, prepared)

    def save_form_data(self, instance, data):
        old_images = set(instance.images)
        new_images = set(data)

        added_images = new_images - old_images
        for item in added_images:
            old_path = default_storage.path(item["file"])
            ext = old_path.split(".")[-1]
            new_file_name = f"{self.upload_to}/{item['uuid']}.{ext}"
            new_path = default_storage.path(new_file_name)
            shutil.move(old_path, new_path)
            item["file"] = new_file_name

        removed_images = old_images - new_images
        for item in removed_images:
            try:
                default_storage.delete(item["file"])
            except FileNotFoundError:
                pass

        return super().save_form_data(instance, data)
